/*
 * BTree.java
 * 
 * Copyright (C) 1996 Paul Phillips <paulp@cerf.net>.
 * Permission granted for any use as long as this header is maintained.
 */

package btree;

/**
 * A generic binary search tree.  The insert, delete, and find methods
 * will throw an IllegalComparisonException if an attempt is made to insert
 * an element of a type not already in the tree.  Elements must implement
 * the BTreeElement interface.
 */

public class BTree {
    private BTreeNode		root;	// pointer to root BTreeNode
    private int			size;	// number of elements in tree

    /**
     * Constructs a new binary tree.
     */

    public BTree() {
	root = null;
	size = 0;
    }

    /** 
     * Inserts an element into the tree.
     * @param data	element to be inserted
     */

    public void insert(BTreeElement data) 
    throws IllegalComparisonException {
	if(root == null) {
	    root = new BTreeNode(data);
	    size++;
	}
	else {
	    if(root.getElementClass() != data.getClass())
		throw new IllegalComparisonException();
	    else {
		root = root.insert(data);
		size++;
	    }
	}
    }

    /**
     * Deletes an element from the tree.  Returns true if it was
     * successfully deleted, false if it did not exist.
     * @param data	element to be deleted
     */

    public boolean delete(BTreeElement data) 
    throws IllegalComparisonException {
	if(root == null)
	    return false;

	if(root.getElementClass() != data.getClass())
	    throw new IllegalComparisonException();
	else {
	    try {
		root = root.delete(data);
	    }
	    catch (ElementNotFoundException e) {
		return false;
	    }
	    size--;
	    return true;
	}
    }

    /**
     * Finds an element in the tree, without modifying it.  Returns
     * null if the element does not exist.
     * @param data	element to be found
     */

    public BTreeElement find(BTreeElement data) 
    throws IllegalComparisonException {
	if(root == null)
	    return null;
	else {
	    if(root.getElementClass() != data.getClass())
		throw new IllegalComparisonException();
	    else
		return root.find(data);
	}
    }

    /**
     * Query the number of elements in the tree.
     */
 
    public int size() {
	return size;
    }

    /** 
     * Print all the elements in the tree.  Useful for debugging.
     */

    public void printAll() {
	if(root == null)
	    System.out.println("Empty BTree");
	else
	    root.printAll();
    }
}

