/*
 * BTreeNode.java
 *
 * Copyright (C) 1996 Paul Phillips <paulp@cerf.net>.
 * Permission granted for any use as long as this header is maintained.
 */

package btree;

/**
 * The BTreeNode class holds an element and points to its child trees.
 * This class should not be used directly! Use the BTree wrapper class
 * instead.  It's not documented to encourage you.  (No, I just haven't
 * gotten around to it.)
 */

class BTreeNode {
    private BTreeElement	data;
    private BTreeNode		left = null;
    private BTreeNode		right = null;

    public BTreeNode(BTreeElement data) {
	this.data = data;
    }

    BTreeNode insert(BTreeElement data) {
	if(data.lessThan(this.data))
	    left = (left == null) ? new BTreeNode(data) : left.insert(data);
	else
	    right = (right == null) ? new BTreeNode(data) : right.insert(data);

	return this;
    }

    BTreeElement find(BTreeElement data) {
	if(data.equalTo(this.data))
	    return this.data;

	if(data.lessThan(this.data))
	    return (left == null) ? null : left.find(data);
	else
	    return (right == null) ? null : right.find(data);
    }

    BTreeNode delete(BTreeElement data) 
    throws ElementNotFoundException {
	if(data.equalTo(this.data))
	    return join(left, right);

	if(data.lessThan(this.data)) {
	    if(left == null)
		throw new ElementNotFoundException();
	    else
		left = left.delete(data);
	}
	else {
	    if(right == null)
		throw new ElementNotFoundException();
	    else
		right = right.delete(data);
	}

	return this;
    }

    private BTreeNode join(BTreeNode left, BTreeNode right) {
	BTreeNode newtree, searcher, trailer = null;

	if(left == null)
	    return right;
	if(right == null)
	    return left;

	searcher = right;

	if(searcher.left == null) {
	    newtree = new BTreeNode(searcher.data);
	    newtree.left = left;
	    newtree.right = searcher.right;
	    return newtree;
	}

	while(searcher.left != null) {
	    trailer = searcher;
	    searcher = searcher.left;
	}

	newtree = new BTreeNode(searcher.data);
	newtree.left = left;
	newtree.right = searcher.right;

	trailer.left = null;

	if(newtree.right == null) {
	    newtree.right = right;
	}
	else {
	    searcher = newtree.right;
	    while(searcher.right != null)
		searcher = searcher.right;

	    searcher.right = right;
	}

	return newtree;
    }

    Class getElementClass() {
	return data.getClass();
    }

    void printAll() {
	if(left != null)
	    left.printAll();
	data.print();
	if(right != null)
	    right.printAll();
    }
}

