/*
 * TestTree.java
 *
 * Copyright (C) 1996 Paul Phillips <paulp@cerf.net>.
 * Permission granted for any use as long as this header is maintained.
 */

package btree;

/**
 * Tests the binary search tree.  Defines a simple element class, 
 * instantiates ten of them, then deletes five and prints out both
 * the original ten and what's left in the BTree.  Undocumented.
 */

import java.util.Random;

public class TestTree {
    static final int NUM_ELEMENTS = 10;
    static final int NUM_RANGE = 10000;
    static Random rand = new Random();

    public static void main(String argv[]) {
	int i;
	StringBuffer s;
	IntElement[] elements = new IntElement[NUM_ELEMENTS];
	BTree tree = new BTree();

	/* Create array and BTree with the same elements */
	for(i = 0; i < NUM_ELEMENTS; i++) {
	    elements[i] = new IntElement(Math.abs(rand.nextInt()) % NUM_RANGE);
	    try {
	    	tree.insert(elements[i]);
	    }
	    catch(IllegalComparisonException e) { }
	}

	tree.printAll();

	/* Delete even indexed elements from btree */
	for(i = 0; i < NUM_ELEMENTS; i += 2) {
	    try {
		tree.delete(elements[i]);
	    }
	    catch(IllegalComparisonException e) { }
	}

	/* Print array elements */
	s = new StringBuffer("Array elements:");
	for(i = 0; i < NUM_ELEMENTS; i++)
	    s.append(" " + elements[i].toString());
	
	System.out.println(s);

	/* Print BTree contents */
	System.out.println("BTree Contents:");
	tree.printAll();
    }
}

class IntElement implements BTreeElement {
    int value;		/* Random value */

    public IntElement(int value) {
	this.value = value;
    }

    public boolean lessThan(BTreeElement data) {
	if(value < ((IntElement)data).value)
	    return true;
	else
	    return false;
    }

    public boolean equalTo(BTreeElement data)  {
	if(value == ((IntElement)data).value)
	    return true;
	else
	    return false;
    }

    public void print() {
	System.out.println(value);
    }

    public String toString() {
	return Integer.toString(value);
    }
}

